package com.adc.da.system.service.impl;


import com.adc.da.dto.resDTO.DownCheckFalseDTO;
import com.adc.da.dto.resDTO.ImportCheckResultResDTO;
import com.adc.da.system.dao.CheckDataCheckResultMapper;
import com.adc.da.system.entity.CheckDataCheckResult;
import com.adc.da.system.service.ICheckDataCheckResultService;
import com.adc.da.util.ReadExcel;
import com.adc.da.excel.poi.excel.ExcelExportUtil;
import com.adc.da.excel.poi.excel.entity.ExportParams;
import com.adc.da.excel.poi.excel.entity.enums.ExcelType;
import com.adc.da.util.utils.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.poi.ss.usermodel.Workbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-10-17
 */
@Service
public class CheckDataCheckResultServiceImpl extends ServiceImpl<CheckDataCheckResultMapper, CheckDataCheckResult>
        implements ICheckDataCheckResultService {
    private static final Logger logger = LoggerFactory.getLogger(ICheckDataCheckResultService.class);

    @Override
    public void delByMainIdAndType(String id, String type) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("CHECK_CERTIFICATE_MAIN_INFO_ID",id);
        wrapper.eq("CHECK_TYPE",type);
        baseMapper.delete(wrapper);
    }

    @Override
    public List<CheckDataCheckResult> selectByIdAndType(String id, String type) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("CHECK_CERTIFICATE_MAIN_INFO_ID",id);
        wrapper.eq("CHECK_TYPE",type);
        wrapper.orderByDesc("CHECK_RESULT");
        return baseMapper.selectList(wrapper);
    }
    @Override
    public List<String> selectCheckOkDetail(String id) {
        return baseMapper.selectCheckOkDetail(id);
    }

    @Override
    public void exportCheckResult(HttpServletResponse response, HttpServletRequest request, String id, String type) {
        Workbook workbook = null;
        try (OutputStream os = response.getOutputStream()) {
            response.setHeader("Content-Disposition",
                    "attachment; filename=" + ReadExcel.encodeFileName
                            (  "随车文件一致性对比详情.xlsx", request));
            response.setContentType("application/force-download");
            ExportParams exportParams = new ExportParams();
            exportParams.setType(ExcelType.XSSF);
            List<CheckDataCheckResult> checkResults = this.selectByIdAndType(id,type);
            List<ImportCheckResultResDTO> data = new ArrayList<>();
            ImportCheckResultResDTO importCheckResultResDTO;
            CheckDataCheckResult result;
            for (int i = 0; i<checkResults.size();i++){
                result = checkResults.get(i);
                importCheckResultResDTO = ImportCheckResultResDTO.builder().index(String.valueOf(i+1))
                        .checkResult("0".equals(result.getCheckResult())? "一致":"不一致")
                        .fromData(result.getCheckFormData())
                        .parameterName(result.getParamName())
                        .parameterValue(result.getCheckData()).build();
                data.add(importCheckResultResDTO);
            }
            workbook = ExcelExportUtil.exportExcel(exportParams, ImportCheckResultResDTO.class, data);
            workbook.write(os);
        } catch (Exception e) {
            logger.error("下载出错");
        }
    }

    @Override
    public IPage getListInfo(int page, int limit, String address, String carTypeCode, String result, String carGroup, String editionCode, String paramName) {
        Page page1 = new Page();
        page1.setCurrent(page).setSize(limit);
        List<String> addresses = new ArrayList<>();
//        if (StringUtils.isNotBlank(address)) {
//            addresses = Arrays.asList(address.split(","));
//        }
        return baseMapper.getListInfo(page1,addresses,carTypeCode,result,carGroup,editionCode,paramName);
    }

    @Override
    public Boolean updateCheckFalse(String id) {
        UpdateWrapper wrapper = new UpdateWrapper();
        wrapper.set("CHECK_RESULT","2");
        wrapper.eq("ID",id);
        wrapper.set("CHECK_DATE",new Date());
        return this.update(wrapper);
    }

    @Override
    public List<DownCheckFalseDTO> downCheckFalse(String address, String project, String isHandel, String carGroup, String editionCode, String paramName) {
        List<String> addresses = new ArrayList<>();
        if (StringUtils.isNotBlank(address)) {
            addresses = Arrays.asList(address.split(","));
        }
        return baseMapper.downCheckFalse(addresses,project,isHandel, carGroup, editionCode, paramName);
    }

    @Override
    public boolean updateResultById(String id, String result) {
        UpdateWrapper wrapper = new UpdateWrapper();
        wrapper.eq("ID",id);
        wrapper.set("CHECK_RESULT",result);
        return this.update(wrapper);
    }
}
