package com.adc.da.system.service.impl;


import com.adc.da.dto.resDTO.CheckEnterpriseDTO;
import com.adc.da.dto.resDTO.CheckEnterpriseDataDto;
import com.adc.da.system.dao.CheckEnterpriseDataDao;
import com.adc.da.system.entity.CheckEnterpriseData;
import com.adc.da.system.entity.VehicleList;
import com.adc.da.system.service.ICheckEnterpriseDataService;
import com.adc.da.util.UUIDUtils;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class CheckEnterpriseDataServiceImpl
        extends ServiceImpl<CheckEnterpriseDataDao, CheckEnterpriseData>
        implements ICheckEnterpriseDataService {


//    @Autowired
//    private ICheckEnterpriseDataDetailsService iCheckEnterpriseDataDetailsService;

    @Override
    public IPage<CheckEnterpriseData> queryCheckEnterpriseDataByPage(Page page,
                                                                     CheckEnterpriseData checkEnterpriseData) {
        if (checkEnterpriseData.getOrderBy() != null) {
            switch (checkEnterpriseData.getOrderBy()) {
                case "cccId":
                    checkEnterpriseData.setOrderBy("CCC_ID");
                    break;
                case "vehicleType":
                    checkEnterpriseData.setOrderBy("VEHICLE_TYPE");
                    break;
                case "versionNumber":
                    checkEnterpriseData.setOrderBy("VERSION_NUMBER");
                    break;
                default:
                    break;
            }
        }
        List<String> baseIdList = new ArrayList<>();
        if (checkEnterpriseData.getBaseId() != null){
            baseIdList = Arrays.asList(checkEnterpriseData.getBaseId().split(","));
        }
        return baseMapper.queryCheckEnterpriseDataByPage(page, checkEnterpriseData,baseIdList);
    }
    @Override
    public IPage<VehicleList> queryVehicleListByPage(Page page, VehicleList vehicleList) {
        if (vehicleList.getOrderBy() != null) {
            switch (vehicleList.getOrderBy()) {
                case "vehicleModel":
                    vehicleList.setOrderBy("VEHICLE_MODEL");
                    break;
                case "productNo":
                    vehicleList.setOrderBy("PRODUCT_NO");
                    break;
                case "vehicleName":
                    vehicleList.setOrderBy("NLSSORT(VEHICLE_NAME,'NLS_SORT = SCHINESE_PINYIN_M')");
                    break;
                case "vehicleGroup":
                    vehicleList.setOrderBy("to_number(VEHICLE_GROUP)");
                    break;
                case "cccSystemNumber":
                    vehicleList.setOrderBy("CCC_SYSTEM_NUMBER");
                    break;
                case "envPublicNumber":
                    vehicleList.setOrderBy("ENV_PUBLIC_NUMBER");
                    break;
                case "editionCode":
                    vehicleList.setOrderBy("to_number(EDITION_CODE)");
                    break;
                default:
                    vehicleList.setOrderBy(null);
            }
        }
        List<String> baseIdList = new ArrayList<>();
//        if (vehicleList.getBaseId() != null){
//            baseIdList = Arrays.asList(vehicleList.getBaseId().split(","));
//        }
        return baseMapper.queryVehicleListByPage(page, vehicleList,baseIdList);
    }

    @Override
    public Map<String, Object> importCheckEnterpriseData(MultipartFile file, String founder) {
        List<CheckEnterpriseDataDto> excelList = new ArrayList<>();
        //文件流
        String fileName = file.getOriginalFilename();
        InputStream is = null;
        try {
            is = file.getInputStream();
            if(is == null){
                return null;
            }
            Workbook workbook = null;
            if (fileName.endsWith("xlsx")) {
                workbook = new XSSFWorkbook(is);
            } else {
                workbook = new HSSFWorkbook(is);
            }
            //获取Excel文件第一个sheet
            Sheet sheet = workbook.getSheetAt(0);
            String title[] = {"序号","车辆型号","版本号","备注"};
            if(!(sheet.getRow(0) !=null && title[0].equals(sheet.getRow(0).getCell(0).getStringCellValue())
                    && title[1].equals(sheet.getRow(0).getCell(1).getStringCellValue())
                    && title[2].equals(sheet.getRow(0).getCell(2).getStringCellValue())
                    && title[3].equals(sheet.getRow(0).getCell(3).getStringCellValue()))){
                return null;
            }
            for (int i = 1; i <= sheet.getLastRowNum(); i++) {
                CheckEnterpriseDataDto checkEnterpriseDataDto = new CheckEnterpriseDataDto();
                if (sheet.getRow(i) != null){
                    for (int j = 1 ; j <= sheet.getRow(i).getLastCellNum() ; j++){
                        if (sheet.getRow(i).getCell(j) != null) {
                            sheet.getRow(i).getCell(j).setCellType(Cell.CELL_TYPE_STRING);
                        }
                    }
                    checkEnterpriseDataDto.setVehicleType(String.valueOf(sheet.getRow(i).getCell(1)));
                    checkEnterpriseDataDto.setVersionNumber(String.valueOf(sheet.getRow(i).getCell(2)));
                    checkEnterpriseDataDto.setRemarks(String.valueOf(sheet.getRow(i).getCell(3)));
                    excelList.add(checkEnterpriseDataDto);
                }else {
                    checkEnterpriseDataDto.setVehicleType("");
                    checkEnterpriseDataDto.setVersionNumber("");
                    checkEnterpriseDataDto.setRemarks("");
                    excelList.add(checkEnterpriseDataDto);
                }
            }
        } catch (IOException e) {
            return null;
        }
      return   this.importCheckEnterpriseData2(excelList,founder);

    }

    @Override
    public String getCCAPDataByCode(String certificateCode) {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("CCC_ID",certificateCode);
        CheckEnterpriseData data = this.getOne(wrapper);
        return data!=null ? data.getId():null  ;
    }

    @Override
    public IPage<CheckEnterpriseData> queryInfoByCheckstatus(Page page) {
        return baseMapper.queryInfoByCheckstatus(page);
    }

    @Override
    public IPage<CheckEnterpriseData> queryEnterpriseDataPage(Page page,CheckEnterpriseData checkEnterpriseData) {
        if(StringUtils.isNotBlank(checkEnterpriseData.getOrderBy())){
            switch (checkEnterpriseData.getOrderBy()){
                case "vehicleType":
                    checkEnterpriseData.setOrderBy("NLSSORT(VEHICLE_TYPE,'NLS_SORT = SCHINESE_PINYIN_M')");
                    break;
                case "entryName":
                    checkEnterpriseData.setOrderBy("NLSSORT(ENTRY_NAME,'NLS_SORT = SCHINESE_PINYIN_M')");
                    break;
                case "versionNumber":
                    checkEnterpriseData.setOrderBy("VERSION_NUMBER");
                    break;
                case "checkTime":
                    checkEnterpriseData.setOrderBy("CHECK_TIME");
                    break;
                case "founder":
                    checkEnterpriseData.setOrderBy("NLSSORT(FOUNDER,'NLS_SORT = SCHINESE_PINYIN_M')");
                    break;
                case "remarks":
                    checkEnterpriseData.setOrderBy("NLSSORT(REMARKS,'NLS_SORT = SCHINESE_PINYIN_M')");
                    break;
                default:
                    checkEnterpriseData.setOrderBy("TYPE");
                    checkEnterpriseData.setOrder("asc");
            }
        }else{
//            checkEnterpriseData.setOrderBy("CHECK_RESULT DESC,CHECK_TIME DESC");
            checkEnterpriseData.setOrder("DESC");
            checkEnterpriseData.setOrderBy("CHECK_RESULT ,CHECK_TIME ");
        }
        if (checkEnterpriseData.getOrder() != null && checkEnterpriseData.getOrder().equals("normal")) {
            checkEnterpriseData.setOrder(null);
        }
        return baseMapper.queryEnterpriseDataPage(page,checkEnterpriseData);
    }

    @Override
    public List<CheckEnterpriseDTO> queryEnterpriseData(String vehicleType, String entryName, String checkStatus, String auditNumber, String versionNumber) {
        return baseMapper.queryEnterpriseData(vehicleType,entryName,checkStatus,auditNumber,versionNumber);
    }

//    @Override
//    public Integer updateEnterpriseData(CheckEnterpriseData checkEnterpriseData) {
////        checkEnterpriseData.setCreateTime(new Date());
////        checkEnterpriseData.setCheckStatus("2");
//        UpdateWrapper<CheckEnterpriseDataDetails> queryWrapper = new UpdateWrapper<>();
//        queryWrapper.set("CHECK_RESULT","2");
//        queryWrapper.eq("ID",checkEnterpriseData.getId());
//        iCheckEnterpriseDataDetailsService.update(queryWrapper);
//        return 0;
//    }

    private Map<String, Object> importCheckEnterpriseData2(List<CheckEnterpriseDataDto> excelList, String founder) {
        List<CheckEnterpriseData> successList = new ArrayList<>();
        List<String> errorList = new ArrayList<>();
        List<String> isNullList = new ArrayList<>();
        List<String> vehicleType = new ArrayList<>();
        List<String> versionNumber = new ArrayList<>();
        try {
            for (int i = 0; i < excelList.size(); i++) {
                if (((StringUtils.isNotEmpty(excelList.get(i).getVehicleType()) && !excelList.get(i).getVehicleType().equals("null"))
                        && (StringUtils.isNotEmpty(excelList.get(i).getVersionNumber()) && !excelList.get(i).getVersionNumber().equals("null"))) && !(excelList.get(i).getVersionNumber().length()>30||excelList.get(i).getVehicleType().length()>30 || excelList.get(i).getRemarks().length()>30) ) {
                    if (vehicleType.contains(excelList.get(i).getVehicleType())
                            && versionNumber.contains(excelList.get(i).getVersionNumber())) {
                        errorList.add(String.valueOf(i + 1));
                        continue;
                    } else {
                        vehicleType.add(excelList.get(i).getVehicleType());
                        versionNumber.add(excelList.get(i).getVersionNumber());
                    }
                    QueryWrapper queryWrapper = new QueryWrapper();
                    queryWrapper.eq("VEHICLE_TYPE", excelList.get(i).getVehicleType());
                    queryWrapper.eq("VERSION_NUMBER", excelList.get(i).getVersionNumber());
                    List<CheckEnterpriseData> list = baseMapper.selectList(queryWrapper);
                    this.importCheckEnterpriseData3(list,errorList,i,excelList,founder,successList);

                } else {
                    isNullList.add(String.valueOf(i + 1));
                }
            }
            if (this.saveBatch(successList)) {
                Map<String, Object> map = new HashMap<>();
                map.put("successList", successList);
                map.put("errorList", errorList);
                map.put("isNullList", isNullList);
                return map;
            } else {
                return null;
            }
        } catch (Exception e) {
            return null;
        }
    }

    private void importCheckEnterpriseData3(List<CheckEnterpriseData> list, List<String> errorList, int i, List<CheckEnterpriseDataDto> excelList, String founder, List<CheckEnterpriseData> successList) {
        if (!list.isEmpty()) {
            errorList.add(String.valueOf(i + 1));
        } else {
            CheckEnterpriseData checkEnterpriseData = new CheckEnterpriseData();
            checkEnterpriseData.setVehicleType(excelList.get(i).getVehicleType());
            checkEnterpriseData.setVersionNumber(excelList.get(i).getVersionNumber());
            checkEnterpriseData.setFounder(founder);
            checkEnterpriseData.setRemarks(excelList.get(i).getRemarks() == "null" ? "":excelList.get(i).getRemarks());
            if (successList.contains(checkEnterpriseData)) {
                errorList.add(String.valueOf(i + 1));
            } else {
                checkEnterpriseData.setId(UUIDUtils.getUUID());
                checkEnterpriseData.setCreateTime(new Date());
                successList.add(checkEnterpriseData);
            }
        }
    }
}
